<?php
namespace WP_SYN;

class LOCAL_CONTENT{
    var $cpt_syn_post;
    var $cron_hook_name;
    var $last_item_id;
    var $last_item_id_option;

    var $Taxonomies;

    function __construct(){
        add_action('init', array($this, 'init'));

        // Post
        add_filter( 'use_block_editor_for_post_type', array($this, 'disable_gutenberg'), 10, 2 );
        add_filter( 'the_title', array($this, 'the_title'), 10, 2 );
        add_action('wpsyn_save_post_after', array($this, 'add_metadata'), 10, 2);
        add_action('wpsyn_save_post_after', array($this, 'add_media'), 11, 2);
        add_action('add_meta_boxes', array(&$this, 'add_custom_box'));
		    add_action('save_post', array(&$this, 'save_postdata'), 11, 3);

        // Cron
        add_action('wpsyn_settings_page', array($this, 'settings_page'));
        $this->cron_hook_name = 'wpsyn_cron_fetch_posts';
        add_action( $this->cron_hook_name , array($this, 'execute_cron') );

        include_once plugin_dir_path(__FILE__) . 'local-taxonomies.php';
        $this->Taxonomies = new LOCAL_TAXONOMIES;
    }

    public function init(){
        if(!isset(wp_syn()->Settings->Options()['local_storage']) || wp_syn()->Settings->Options()['local_storage'] !== 'yes'){
            return;
        }
        $this->last_item_id_option = 'wpsyn_last_item_id';
        $this->last_item_id = get_option($this->last_item_id_option);
        $this->cpt_syn_post = apply_filters('wpsyn_post_type', 'syn_post');
        if($this->cpt_syn_post === 'syn_post'){
            $cpt_syn_post_params = apply_filters('wpsyn_post_type_params', include 'cpt/syn-post.php');
            register_post_type($this->cpt_syn_post, $cpt_syn_post_params);
            do_action('wpsyn_after_register_post_type', $this->cpt_syn_post);
        }
        add_filter('manage_'.$this->cpt_syn_post.'_posts_columns', array(&$this, 'columns_head'), 2);
        add_action('manage_'.$this->cpt_syn_post.'_posts_custom_column', array(&$this, 'columns_content'), 1, 2);
        do_action('wpsyn_after_init', $this);

    }

    public function the_title($title, $post_id) {
        $post = get_post($post_id);
        if($post->post_type === $this->cpt_syn_post && empty($title)){
            $title = substr(strip_tags($post->post_content), 0, 50);
        }
        return $title;
    }

    /**
    * alters columns
    * @param array $defaults
    * @return array
    * @filter eventpost_columns_head
    */
    public function columns_head($defaults) {
        $defaults['network'] = __('Network', 'wp-slideyournet');
        $defaults['user'] = __('User', 'wp-slideyournet');
        return $defaults;
    }

    /**
    * echoes content of a row in a given column
    * @param string $column_name
    * @param int $post_id
    * @action eventpost_columns_content
    */
    public function columns_content($column_name, $post_id) {
        if ($column_name == 'network') {
            $network = get_post_meta($post_id, 'syn_network', true);
            echo '<span class="dashicons dashicons-'.$network.'"></span> '.ucfirst($network);
        }
        if ($column_name == 'user') {
            echo '@'.get_post_meta($post_id, 'syn_username', true);
        }
    }

    /**
     * add custom boxes in posts edit page
     */
    public function add_custom_box() {
        add_meta_box(
            'wpsyn_metas',
            __('Meta data', 'wp-slideyournet'),
            array(&$this, 'inner_custom_box'),
            $this->cpt_syn_post,
            'normal',
            'core',
            array(
                '__block_editor_compatible_meta_box' => true,
            )
        );
    }

    public function inner_custom_box($post) {
        wp_nonce_field(plugin_basename(__FILE__), 'wpsyn_local_content');
        include (plugin_dir_path(__DIR__) . 'views/admin/local-custombox.php');
    }


    public function disable_gutenberg($current_status, $post_type ) {
        if($post_type === 'syn_post'){
            return false;
        }
        return $current_status;
    }

    function posted_meta_value($meta_key, $type='post'){
        $meta_value = filter_input(INPUT_POST, 'wpsyn-'.$meta_key);
        return sanitize_meta( 'syn_'.$meta_key, sanitize_text_field($meta_value), $type );
    }

    /**
     * When the post is saved, saves our custom data
     * @param int $post_id
      * @return void
    */
    public function save_postdata($post_id, $post, $update) {
        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE){
            return;
        }

        if (!wp_verify_nonce(filter_input(INPUT_POST, 'wpsyn_local_content'), plugin_basename(__FILE__))){
            return;
        }

        $network = $this->posted_meta_value('network');
        update_post_meta($post_id, 'syn_network', $network);

        $username = $this->posted_meta_value('username');
        update_post_meta($post_id, 'syn_username', $username);

        $syn_rawdata_uptodate = get_post_meta($post_id, 'syn_rawdata_uptodate', true);
        if(!$syn_rawdata_uptodate){
          $syn_rawdata_uptodate = (array) get_post_meta($post_id, 'syn_rawdata', true);
        }
        $syn_rawdata_uptodate["network"] = $network;
        $syn_rawdata_uptodate["content"] = $post->post_content;
        $syn_rawdata_uptodate["user"]["name"] = $username;
        $syn_rawdata_uptodate["media_url"] = get_post_thumbnail_id($post_id) ? get_post_thumbnail_id($post_id) : "";

        update_post_meta($post_id, 'syn_rawdata_uptodate', apply_filters( 'wpsyn_update_rawdata', $syn_rawdata_uptodate ,$post_id ));
    }

    /**
     * Insert a local post from remote data
     * @param  object $post
     * @return $post_id ID of the inserted post
     */
    public function save_post($post){
        if(!$this->cpt_syn_post){
            return;
        }
        $title = sprintf(_x('%1$s on %2$s: %3$s', 'Post title. fill with username, network, content.', 'wp-slideyournet'), $post->user->name, $post->network, substr(strip_tags($post->content), 0, 20));
        $post_data = apply_filters('wpsyn_save_post', array(
                'post_type'=>$this->cpt_syn_post,
                'post_title'=>$title,
                'post_content'=>$post->content."\n\n".$post->pub_url,
                'post_date'=>$post->pub_date,
                'post_status'=>'publish',
        ), $post);

        $post_id = wp_insert_post($post_data, false, false);
        do_action('wpsyn_save_post_after', $post_id, $post);
        return $post_id;
    }

    /**
     * Add meta data to an insterted post
     * @param int $post_id
     * @param object $post
     */
    public function add_metadata($post_id, $post){
        // Network
        update_post_meta($post_id, 'syn_network', $post->network);

        // User
        update_post_meta($post_id, 'syn_username', $post->user->name);
        update_post_meta($post_id, 'syn_useravatar', $post->user->avatar_url);


        // Geo data
        if(!empty($post->pub_lat) && !empty($post->pub_lon)){
            // http://codex.wordpress.org/Geodata
            update_post_meta($post_id, 'geo_latitude', $post->pub_lat);
            update_post_meta($post_id, 'geo_longitude', $post->pub_lon);
        }

        // Raw data
        $post_array  = (array) $post;
        $post_array['user'] = (array) $post_array['user'];
        update_post_meta($post_id, 'syn_rawdata', $post_array);
        update_post_meta($post_id, 'syn_rawdata_uptodate', $post_array);
    }

    function add_media($post_id, $post){
        if(empty($post->media_url)){
            return;
        }
        $file_content = \WP_SYN\UTILS::get_remote_content($post->media_url);
        if(!$file_content){
            error_log('WPSYN: Media content could not be fetched');
            return;
        }
        $ext = 'jpg';
        $filename = "syn-{$post->id}.{$ext}";
        $filetype = \wp_check_filetype($filename);
        $wp_upload_dir = \wp_upload_dir();
        $filepath = $wp_upload_dir['path'].'/'.$filename;


        $title = sprintf(_x('%2$s\'s image on %1$s', 'Media title. fill with network and username.', 'wp-slideyournet'), $post->network, $post->user->name);

        $attachment = array(
            'guid'           => $wp_upload_dir['url'] . '/' . $filename,
            'post_mime_type' => $filetype['type'],
            'post_title'     => $title,
            'post_content'   => $post->content,
            'post_status'    => 'inherit'
        );

        file_put_contents($filepath, $file_content);
        $attachment_id = \wp_insert_attachment( $attachment, $filepath, $post_id );
        require_once( ABSPATH . 'wp-admin/includes/image.php' );
        $attach_data = \wp_generate_attachment_metadata( $attachment_id, $filepath );
        \wp_update_attachment_metadata( $attachment_id, $attach_data );
        \set_post_thumbnail($post_id, $attachment_id);
        $syn_rawdata = get_post_meta($post_id, 'syn_rawdata', true);
        $syn_rawdata["media_url"] = $attachment_id;
        update_post_meta($post_id, 'syn_rawdata', apply_filters( 'wpsyn_add_attachment_rawdata', $syn_rawdata ,$post_id,$post ));
        update_post_meta($post_id, 'syn_rawdata_uptodate', apply_filters( 'wpsyn_add_attachment_rawdata', $syn_rawdata ,$post_id,$post ));
        return $attachment_id;
    }


    public function execute_cron(){
        $params = array(
            'order_by'=>'id',
            'order'=>'ASC',
            // 'per_page'=>5,
            'minid' => $this->last_item_id,
        );
        $posts = wp_syn()->get_posts($params);
        if(!$posts){
            return;
        }
        foreach($posts as $post){
            $this->save_post($post);
        }
        update_option($this->last_item_id_option, $post->id);
    }

    public function settings_page(){
        if(wp_syn()->Settings->Options()['local_storage']==='yes'){
            if (! wp_next_scheduled ( $this->cron_hook_name )) {
                wp_schedule_event( time(), 'hourly', $this->cron_hook_name  );
                echo "<div class='notice notice-success settings-error is-dismissible'><p><strong>" . __( 'Scheduled tasks have been enabled.', 'wp-slideyournet' ) . '</strong></p></div>';
            }
        }
        else{
            if (wp_next_scheduled ( $this->cron_hook_name  )) {
                wp_clear_scheduled_hook( $this->cron_hook_name );
                echo "<div class='notice notice-success settings-error is-dismissible'><p><strong>" . __( 'Scheduled tasks have been disabled.', 'wp-slideyournet' ) . '</strong></p></div>';
            }
        }
    }


}
