<?php
namespace WP_SYN;

class LOCAL_TAXONOMIES{
    var $cpt_syn_tag;
    var $tags_to_terms;

    function __construct(){
        add_action('wpsyn_after_register_post_type', array(&$this, 'register_ct'));
        add_action('wpsyn_after_init', array(&$this, 'add_hooks'));
        add_action('wpsyn_save_post_after', array($this, 'add_terms'), 11, 2);
    }

    public function register_ct($post_type){
        $this->cpt_syn_tag = apply_filters('wpsyn_tag', 'syn_tag');
        $this->cpt_syn_tag_params = apply_filters('wpsyn_tag_params', include 'cpt/syn-tag.php');
        register_taxonomy($this->cpt_syn_tag,
            $post_type,
            $this->cpt_syn_tag_params
        );
    }

    public function columns_header($columns){
        $columns['syn_hashtags'] = __('Hashtags', 'wp-slideyournet');
        return $columns;
    }

    public function columns_content($deprecated, $column_name, $term_id){
        if( $column_name == 'syn_hashtags' ){
            $hashtags = explode(',', get_term_meta($term_id, 'syn_hashtag', true));
            echo implode(', ', array_map(function($h){ return '#'.$h; }, $hashtags));
        }
    }

    /**
     * Get list of mapped hashtag/term
     * @return array
     */
    public function tags(){
        if(!$this->tags_to_terms){
            $this->tags_to_terms = array();
            global $wpdb;
            $query = "SELECT
                m.`meta_value` as hashtag,
                m.`term_id`,
                t.`taxonomy`
            FROM
                {$wpdb->prefix}termmeta m
                JOIN {$wpdb->prefix}term_taxonomy t ON m.`term_id`=t.`term_id`
            WHERE
                `meta_key`='syn_hashtag'";
            $tag_result = $wpdb->get_results($query);
            foreach($tag_result as $tag){
                $single_tags = explode(',', $tag->hashtag);
                foreach($single_tags as $single_tag){
                    $this->tags_to_terms[strtolower($single_tag)] = $tag;
                }
            }
        }
        return $this->tags_to_terms;
    }

    /**
     * Add hooks
     * @param \WP_SYN\LOCAL_CONTENT $LOCAL
     */
    public function add_hooks($LOCAL){
        $taxonomies = get_taxonomies(array(
            'object_type'=>array($LOCAL->cpt_syn_post),
        ));
        foreach($taxonomies as $taxonomy){
            add_action($taxonomy.'_add_form_fields', array(&$this,'add'));
            add_action($taxonomy.'_edit_form_fields', array(&$this,'edit'));
            add_action('edited_'.$taxonomy, array(&$this,'save'));
            add_action('create_'.$taxonomy, array(&$this,'save'));
            add_filter("manage_edit-{$taxonomy}_columns", array(&$this, 'columns_header') );
            add_filter("manage_{$taxonomy}_custom_column", array(&$this, 'columns_content'), 10, 3 );
        }
    }

    /**
     *  Save options
     * @param  int $id
     * @return void
     */
    function save($id) {
        if(isset($_POST['wpsyn-hashtag']) ){
            $hashtag = wp_syn()->LocalContent->posted_meta_value('hashtag', 'term');
            update_term_meta($id, 'syn_hashtag', $hashtag);
        }
    }

    /**
     * add to the add form
     * @param string $term
     * @return void
     */
    function add($term){
        ?>
        <div class="form-field term-description-wrap">
        	<label for="tag-description"><?php _e('Social Media Hashtags','wp-slideyournet'); ?></label>
            #<input type="text" id="hashtag" name="wpsyn-hashtag" value="" class="widefat"/>
            <p class="description">
                <?php _e('Separated by comma.','wp-slideyournet'); ?>
            </p>
            <p class="description">
                <?php _e('Without hashtag prefix.','wp-slideyournet'); ?>
            </p>
        </div>
        <?php
    }

    /**
     *  add to the edit form
     * @param  WP_Term $term
     * @return void
     */
    function edit($term){
        $hastag = get_term_meta($term->term_id, 'syn_hashtag', true);

        ?>
        <table id="sortableTableNaN" class="form-table">
                <tbody>
                        <tr class="form-field form-required">
                                <th valign="top" scope="row">
                                    <label for="hashtag"><?php _e('Social Media Hashtags','wp-slideyournet'); ?></label>
                                </th>
                                <td>
                                    #<input type="text" id="hashtag" name="wpsyn-hashtag" value="<?php esc_attr_e($hastag); ?>" class="widefat"/>
                                    <p class="description">
                                        <?php _e('Separated by comma.','wp-slideyournet'); ?>
                                    </p>
                                    <p class="description">
                                        <?php _e('Without hashtag prefix.','wp-slideyournet'); ?>
                                    </p>
                                </td>
                        </tr>
                </tbody>
        </table>
        <?php
    }

    /**
     * Add terms to a post if needed
     * @param int $post_id  id of a WP_Post
     * @param object $post  SYN post fetched by the API
     */
    public function add_terms($post_id, $post){
        $tags = $this->tags();
        if(empty($tags)){
            return;
        }

        // Get hashtags in post content
        preg_match_all('/(^|[ \.\,\;\:\s])#([\w0-9\/\._]*)/u', strtolower($post->content), $match_tags);

        if($match_tags && isset($match_tags[2]) && is_array($match_tags[2]) && count($match_tags[2])){
            foreach($match_tags[2] as $hashtag){
                $hastag = strtolower($hashtag);
                if(isset($tags[$hashtag])){
                    $tag = $tags[$hashtag];
                    wp_set_post_terms( $post_id, array((int) $tag->term_id), $tag->taxonomy, true );
                }
            }
        }
    }

}
