<?php
namespace WP_SYN;

class SETTINGS{
    var $options;
    var $default_url;

    function __construct(){
        add_action('admin_menu', array(&$this, 'admin_menu'));
        add_action('admin_init', array(&$this, 'admin_init'));
        add_action('init', array(&$this, 'init'), 11);
        $this->default_url = 'https://api.slideyour.net/api/';
    }

    function init(){
        if (wp_syn()->is_network_installed) {
            add_action('network_admin_menu', array(&$this, 'network_admin_menu'));

            add_action(
                'network_admin_edit_wpsynmu',
                array($this, 'trigger_wpsynmu')
            );
        }
        add_filter('plugin_action_links_'.wp_syn()->plugin_file_name, array( &$this, 'settings_link' ) );
    }

    function Options(){
        if(!$this->options){
            $this->options = (array) get_option('wpsyn');
            if (wp_syn()->is_network_installed) {
                $net_options = (array) get_site_option('wpsynmu');
                $this->options = array_merge($this->options, $net_options);
                $this->options['network'] = $net_options;
            }
        }
        if(empty($this->options['url'])){
            $this->options['url']=$this->default_url;
        }
        return $this->options;
    }

    function admin_menu() {
        add_options_page(__('SlideYourNet settings', 'wp-slideyournet'), __('SlideYourNet', 'wp-slideyournet'), 'manage_options', 'syn-settings', array(&$this, 'manage_settings'));
    }
    function network_admin_menu(){
        add_submenu_page('settings.php', __('SlideYourNet global settings', 'wp-slideyournet'), __('SlideYourNet', 'wp-slideyournet'), 'manage_network_options', 'syn-network-settings', array(&$this, 'manage_network_settings'));
    }
    function admin_init(){
        register_setting( 'syn-settings', 'wpsyn' );

        add_settings_section(
            'syn-settings-credential',
            __('Credentials', 'wp-slideyournet'),
            array(&$this, 'settings_section_callback'),
            'syn-settings'
        );
        add_settings_section(
            'syn-settings-advanced',
            '',
            array(&$this, 'settings_section_callback'),
            'syn-settings'
        );
        add_settings_section(
            'syn-settings-advanced-end',
            '',
            array(&$this, 'settings_section_callback'),
            'syn-settings'
        );

        add_settings_field(
                'wpsyn_url',
                __('API URL', 'wp-slideyournet'),
                array(&$this, 'settings_field_default_callback'),
                'syn-settings',
                'syn-settings-advanced',
                array( 'name' => 'url', 'description'=>sprintf(__('Leave empty for default value: %s', 'wp-slideyournet'), $this->default_url) )
        );

        add_settings_field(
                'wpsyn_name',
                __('Name of your instance', 'wp-slideyournet'),
                array(&$this, 'settings_field_default_callback'),
                'syn-settings',
                'syn-settings-credential',
                array( 'name' => 'name', 'description'=>__('https://slideyour.net/<code>the_name</code>, if you do not have a session yet, <a href="http://slideyour.net/" target="_blank">create a new one</a>', 'wp-slideyournet') )
        );

        add_settings_field(
                'wpsyn_token',
                __('Token', 'wp-slideyournet'),
                array(&$this, 'settings_field_default_callback'),
                'syn-settings',
                'syn-settings-credential',
                array( 'name' => 'token' )
        );

        add_settings_field(
            'wpsyn_local_storage',
            __('Local Storage', 'wp-slideyournet'),
            array(&$this, 'settings_field_select_callback'),
            'syn-settings',
            'syn-settings-advanced',
            array( 'name' => 'local_storage', 'options'=>array('no'=>__('No', 'wp-slideyournet'), 'yes'=>__('Yes', 'wp-slideyournet')), 'description'=>__('Should publications be stored locally or not', 'wp-slideyournet') )
        );

        add_settings_field(
                'wpsyn_cache',
                __('Cache', 'wp-slideyournet'),
                array(&$this, 'settings_field_default_callback'),
                'syn-settings',
                'syn-settings-advanced',
                array( 'name' => 'cache', 'description'=>__('Number of minutes of result caching (improves performances, default 15, 0 disables cache)', 'wp-slideyournet')  )
        );

        add_settings_field(
                'wpsyn_timeout',
                __('API Timeout', 'wp-slideyournet'),
                array(&$this, 'settings_field_default_callback'),
                'syn-settings',
                'syn-settings-advanced',
                array( 'name' => 'timeout', 'description'=>__('Maximum number of seconds before abording request. (leave empty to use php.ini setting)', 'wp-slideyournet')  )
        );

        if (wp_syn()->is_network_installed) {
            register_setting( 'syn-network-settings', 'wpsynmu' );

            add_settings_section(
            'syn-settings-network-basics',
                __('Global basics', 'wp-slideyournet'),
                array(&$this, 'settings_section_callback'),
                'syn-network-settings'
            );

            add_settings_field(
                    'wpsyn_global_url',
                    __('Global API URL', 'wp-slideyournet'),
                    array(&$this, 'settings_field_default_callback'),
                    'syn-network-settings',
                    'syn-settings-network-basics',
                    array( 'name' => 'url', 'network'=>true, 'description'=>__('example: https://api.slideyour.net/api/', 'wp-slideyournet') )
            );

            add_settings_field(
                    'wpsyn_global_cache',
                    __('Global Cache', 'wp-slideyournet'),
                    array(&$this, 'settings_field_default_callback'),
                    'syn-network-settings',
                    'syn-settings-network-basics',
                    array( 'name' => 'cache', 'network'=>true, 'description'=>__('Number of minutes of result caching (improves performances, default 15, 0 disables cache)', 'wp-slideyournet') )
            );

            add_settings_field(
                    'wpsyn_global_timeout',
                    __('Global API Timeout', 'wp-slideyournet'),
                    array(&$this, 'settings_field_default_callback'),
                    'syn-network-settings',
                    'syn-settings-network-basics',
                    array( 'name' => 'timeout', 'network'=>true, 'description'=>__('Maximum number of seconds before abording request. (leave empty to use php.ini setting)', 'wp-slideyournet')  )
            );
        }
    }

    function trigger_wpsynmu(){
        if (!wp_verify_nonce(\filter_input(INPUT_POST, 'wpsyn_settings', FILTER_SANITIZE_STRING), 'wpsyn_settings')) {
            wp_die(__('Security error', 'wp-slideyournet'));
	    }
        $settings = array(
            'url'=>esc_url($_POST['wpsynmu']['url']),
            'cache'=>esc_attr($_POST['wpsynmu']['cache']),
            'timeout'=>esc_attr($_POST['wpsynmu']['timeout']),
        );
        update_site_option( 'wpsynmu', $settings);
        wp_redirect(
            add_query_arg(
                array( 'page' => 'syn-network-settings'),
                (network_admin_url( 'settings.php' ))
            )
        );
        exit;
    }

    /**
     *  Settings link on the plugins page
     */
    public function settings_link( $links ) {
            $settings_link = '<a href="'.admin_url('options-general.php?page=syn-settings').'">' . __( 'Settings', 'wp-slideyournet' ) . '</a>';
            // place it before other links
            array_unshift( $links, $settings_link );
            return $links;
    }

    function manage_settings(){
        do_action('wpsyn_settings_page');
        include (plugin_dir_path(__FILE__) . '../views/admin/settings.php');
    }

    function manage_network_settings(){
        ?>
    <div class="wrap">
        <h2><?php _e('SlideYourNet global settings', 'wp-slideyournet'); ?></h2>
        <?php settings_errors(); ?>
        <form action="edit.php?action=wpsynmu" method="post">
        <?php wp_nonce_field('wpsyn_settings', 'wpsyn_settings'); ?>
        <?php settings_fields( 'syn-network-settings' ); ?>
        <?php do_settings_sections('syn-network-settings'); ?>
        <?php submit_button(); ?>
        </form>
    </div>
    <?php

    }

    function settings_section_callback( $args, $what=null ) {
        if($args['id']==='syn-settings-advanced'):
        ?>
        <details>
            <summary><?php _e('Advanced', 'wp-slideyournet'); ?></summary>
        <?php
        elseif($args['id']==='syn-settings-advanced-end'):
        ?>
        </details>
        <?php
        endif;
    }

    function settings_field_default_callback($args){
        $options = $this->Options();
        $option = 'wpsyn';
        if(isset($args['network']) && $args['network']){
            $option =  'wpsynmu';
        }
        elseif(isset($options['network'][$args['name']])){
            echo '<code>'.$options['network'][$args['name']].'</code>';
            return;
        }
        ?>
        <input name="<?php echo $option; ?>[<?php echo $args['name']; ?>]" id="<?php echo $args['name']; ?>" value="<?php echo (isset($options[$args['name']]) ? $options[$args['name']] : ''); ?>" class="regular-text"/>
        <?php if(isset($args['description']) && $args['description']): ?>
        <div class="description"><?php echo $args['description']; ?></div>
        <?php endif; ?>
        <?php
    }

    function settings_field_select_callback($args){
        if(!isset($args['options']) || !is_array($args['options'])){
            $this->settings_field_default_callback($args);
            return;
        }
        $options = $this->Options();
        $option = 'wpsyn';
        if(isset($args['network']) && $args['network']){
            $option =  'wpsynmu';
        }
        elseif(isset($options['network'][$args['name']])){
            echo '<code>'.$options['network'][$args['name']].'</code>';
            return;
        }
        $current_value = (isset($options[$args['name']]) ? $options[$args['name']] : '');
        ?>
        <select name="<?php echo $option; ?>[<?php echo $args['name']; ?>]" id="<?php echo $args['name']; ?>" class="regular-text">
            <?php foreach ($args['options'] as $value => $label): ?>
            <option value="<?php esc_attr_e($value); ?>" <?php  selected($value, $current_value, true); ?>><?php esc_attr_e($label); ?></option>
            <?php endforeach; ?>
        </select>
        <?php if(isset($args['description']) && $args['description']): ?>
        <div class="description"><?php echo $args['description']; ?></div>
        <?php endif; ?>
        <?php
    }

}
