<?php

/**
 * Bridge to Widget Home Builder
 */
global $WP_SYN_WihoBu;
$WP_SYN_WihoBu = new WP_SYN_WihoBu();

class WP_SYN_WihoBu {

    function __construct() {
        global $wp_syn, $WiHoBuGrid;
        add_action('init', array(&$this, 'init'));

        // Add SYN source to feeds
        add_filter('wihobu_feeds', array($this, 'add_feed'));

        // Subscriptions
        add_filter('wihobu_grid_subscription_syn', array($this, 'get_feed'), 10, 3);
    }

    function init() {

    }

    function add_feed($feeds) {
        global $WiHoBuGrid;
        $feeds['syn'] = $WiHoBuGrid->new_feed(array(
            'name' => __('Slide Your Net', 'wp-slideyournet'),
            'icon'=>'megaphone',
            'settings' => array(
                'media' => array('label' => __('Social media', 'wp-slideyournet'), 'callback' => 'WP_SYN_WihoBu_select_media'),
                'search' => array('label' => __('Search', 'wp-slideyournet')),
                'repost'=> array('label' => __('Re-posts', 'wp-slideyournet'), 'callback' => 'WP_SYN_WihoBu_select_repost'),
                'replies'=> array('label' => __('Replies', 'wp-slideyournet'), 'callback' => 'WP_SYN_WihoBu_select_replies'),
            )
        ));
        return $feeds;
    }

    function get_feed($subscription, $number, $offset=0) {
        global $wp_syn, $WiHoBuGrid, $WPScheduledReadOnly;
        $cards = array();
        $syn_query = array(
            'network' => $subscription['media'],
            'search' => $subscription['search'],
            'per_page' => $number,
            'offset' => $offset,
        );
        if(!empty($subscription['replies'])){
            $syn_query['reply'] = $subscription['replies'];
        }
        if(!empty($subscription['repost'])){
            $syn_query['ref'] = $subscription['repost'];
        }
        if($WPScheduledReadOnly && $WPScheduledReadOnly->is_readonly()){
            $syn_query['before'] = date('Y-m-d H:i:s', $WPScheduledReadOnly->from);
        }
        $items = $wp_syn->get_posts($syn_query);
        foreach ($items as $item) {
            if($WPScheduledReadOnly && $WPScheduledReadOnly->is_readonly()){
                if(strtotime($item->pub_date) > $WPScheduledReadOnly->from){
                    continue;
                }
            }
            $item->media_url = apply_filters('wp_syn_get_thumbnail_url', $item->media_url, $item->network, 'media');
            $item->user->avatar_url = apply_filters('wp_syn_get_thumbnail_url', $item->user->avatar_url, $item->network, 'avatar');
            $card = $WiHoBuGrid->new_card(array(
                        'id' => 'syn-' . $item->id,
                        'title' => '',
                        'text' => $wp_syn->content($item),
                        'date' => strtotime($item->pub_date),
                        'author' => $item->user,
                        'reposter' => isset($item->reposter) ? $item->reposter : Null,
                        'scope' => 'external',
                        'type' => $item->network,
                        'source' => 'syn',
                        'link' => $item->pub_url,
                        'media' => $item->media_url ? '<img src="' . $item->media_url . '" alt=""/>' : '',
                        'media_background' => $item->media_url ? $item->media_url : '',
                        'callback' => array($this, 'card_render'),
            ));
            $card->replied = isset($item->replied_post) ? $item->replied_post : false;
            $card->quoted = isset($item->quoted_post) ? $item->quoted_post : false;
            array_push($cards, $card);
        }
        return $cards;
    }

    function card_username($network, $username){
        if($network=="twitter" || $network=="instagram"){
            $author = "@".$username;
        }
        else{
            $author = $username;
        }
        return $author;
    }

    function card_render($card) {
        global $wp_syn;
        $html = '';
        $date_format = get_option('date_format') . ' ' . get_option('time_format');
        $now = current_time('timestamp');

        $pattern = '/<a (.*?)href=[\"\'](.*?)\/\/(.*?)[\"\'](.*?)>(.*?)<\/a>/i';
        $pattern2 = '/<span>http(.*?)<\/span>/i';

        $result_text = $card->text;
        if($card->replied){
            $result_text='<blockquote class="wp-syn-replied">'.$wp_syn->content($card->replied).'</blockquote>'.$result_text;
        }
        if($card->quoted){
            $result_text='<blockquote class="wp-syn-quoted">'.$wp_syn->content($card->quoted).'</blockquote>'.$result_text;
        }
        $result_text = preg_replace($pattern, '<span>$5</span>', $result_text);
        $result_text = preg_replace($pattern2, '', $result_text);

        $url_reply ="";
        $url_retweet ="";
        if($card->type=="twitter"){
            $table_link = explode('/',$card->link);
            $url_reply = "https://twitter.com/intent/tweet?in_reply_to=".end($table_link);
            $url_retweet = "https://twitter.com/intent/retweet?tweet_id=".end($table_link);
        }

        $displayed_author = $this->card_username($card->type, $card->author->name);
        $html.='<div class="wihobu-card-syn-' . ($card->media ? 'image' : 'text') . '">
                    <a href="' . $card->link . '" target="_blank" rel="noopener">'
                        . ($card->type ?
                                '<div class="card-type">' . $card->type . '</div>' : '')
                        . ($card->media ?
                                '<img class="card-media" src="' . $card->media_background . '"/ alt="">' : '')
                        . (($card->author || $result_text) ?
                                '<div class="card-content">' : '')
                        . ($card->author ?
                                "\n\t" . '<div class="card-author">'
                                . '<img src="' . esc_url($card->author->avatar_url) . '" alt="' . esc_attr($displayed_author) . '">'
                                . '<div><span class="syn-user">' . $displayed_author . '</span>' : '')
                        . ($card->reposter ?
                                "\n\t" . '<span class="card-reposter">'
                                .'<i class="fa fa-retweet" aria-hidden="true"></i> '. sprintf(__('by %s', 'wp-slideyournet'), $this->card_username($card->type, $card->reposter->name))
                                . '</span>' : '')
                        . ($card->replied ?
                                "\n\t" . '<span class="card-reposter">'
                                .'<i class="fa fa-reply" aria-hidden="true"></i> '. sprintf(__('in reply to %s', 'wp-slideyournet'), $this->card_username($card->type, $card->replied->user->name))
                                . '</span>' : '')
                        . ($card->quoted ?
                                "\n\t" . '<span class="card-reposter">'
                                .'<i class="fa fa-quote-left" aria-hidden="true"></i> '. $this->card_username($card->type, $card->quoted->user->name)
                                . '</span>' : '')
                        . ($card->author ?
                                "\n\t" . '</div></div>' : '')

                        . ($card->date ?
                                "\n\t" . '<time datetime="' . date('c', $card->date) . '">' . ($now - $card->date < 2 * DAY_IN_SECONDS ? sprintf(__('%s ago', 'wp-slideyournet'), human_time_diff($now, $card->date)) : date_i18n($date_format, $card->date)) . '</time>' : '')
                        . ($result_text ? "\n\t" . '<div class="card-text">' . $result_text . '</div>' : '')

                        . (($card->author || $result_text) ?
                                '</div>' : '')
                        . '
                    </a>'
                    . ($url_reply ? "\n\t" . '<div class="card-tools">
                        <a href="' . $url_reply . '" class="reply" target="_blank"><i class="fa fa-reply" aria-hidden="true"></i></a>
                        <a href="' . $url_retweet . '" class="retweet" target="_blank"><i class="fa fa-retweet" aria-hidden="true"></i></a>
                        </div>' : '') .'

                </div>';
        return $html;
    }

}

function WP_SYN_WihoBu_select_media($name, $id, $value) {
    global $wp_syn;
    $html = '<select data-name="' . $name . '" id="' . $id . '">'
            . '<option value=""></option>';
    foreach ($wp_syn->networks as $network) {
        $html.='<option value="' . $network . '" ' . selected($value, $network, false) . '>' . ucfirst($network) . '</option>';
    }
    $html.= '</select>';
    return $html;
}

function WP_SYN_WihoBu_select_repost($name, $id, $value) {
    $html = '<select data-name="' . $name . '" id="' . $id . '">'
            . '<option value="">' . __('No filter', 'wp-slideyournet') . '</option>'
            . '<option value="yes" ' . selected($value, 'yes', false) . '>' . __('Only reposts', 'wp-slideyournet') . '</option>'
            . '<option value="no" ' . selected($value, 'no', false) . '>' . __('Exclude reposts', 'wp-slideyournet') . '</option>'
            . '</select>';
    return $html;
}

function WP_SYN_WihoBu_select_replies($name, $id, $value) {
    $html = '<select data-name="' . $name . '" id="' . $id . '">'
            . '<option value="">' . __('No filter', 'wp-slideyournet') . '</option>'
            . '<option value="yes" ' . selected($value, 'yes', false) . '>' . __('Only replies', 'wp-slideyournet') . '</option>'
            . '<option value="no" ' . selected($value, 'no', false) . '>' . __('Exclude replies', 'wp-slideyournet') . '</option>'
            . '</select>';
    return $html;
}
