<?php
/*
 * Base file for widgets
 */
class WP_SYN_Widget extends WP_Widget {
    public $defaults;
    function __construct($widget_id='wpsyn_widget', $widget_name=null, $widget_ops=null) {
        global $wp_syn;

        // Widget settings
        if($widget_name==null){
            $widget_name=__('SlideYourNet Feed', 'wp-slideyournet');
        }
        if($widget_ops==null){
            $widget_ops = array(
                'classname' => 'wpsyn_widget',
                'description' => __('Deprecated. Please use the block instead.', 'wp-slideyournet')
            );
        }

        // Create the widget
        parent::__construct(
                $widget_id,
                $widget_name,
                $widget_ops
        );

        $this->defaults = $wp_syn->criteria;
    }
    // PHP4 caller
    function WP_SYN_Widget(){
       $this->__construct();
    }

    function widget($args, $_instance) {
        global $wp_syn;
        $instance = wp_parse_args( (array) $_instance, $this->defaults );

        $params = $instance;
        unset($params['title']);

        $posts = $wp_syn->get_posts($params);

        if(!count($posts)){
            return;
        }

        echo $args['before_widget'];

        // Load StyleSheet
        if($instance['stylesheet']!='none'){
            if($instance['stylesheet']=='theme'){
                $stylesheet  = get_stylesheet_directory_uri().'/wpsyn.css';
            }
            else{
                $stylesheet  = plugins_url('css/wpsyn.css', __FILE__);
            }
        }
        wp_enqueue_style('wp-syn-'.$instance['stylesheet'], $stylesheet, ['dashicons']);
        include (plugin_dir_path(__FILE__) . 'views/list.php');

        echo $args['after_widget'];
    }

    function update($new_instance, $old_instance) {
       return $new_instance;
    }

    function form($_instance) {
        global $wp_syn;
        $instance = wp_parse_args( (array) $_instance, $wp_syn->criteria);
       ?>
        <p><label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title', 'wp-slideyournet' ) ?></label>
          <input class="widefat" name="<?php echo $this->get_field_name('title'); ?>" id="<?php echo $this->get_field_id('title'); ?>" value="<?php echo $instance['title']; ?>"/>
        </p>

        <p><label for="<?php echo $this->get_field_id('network'); ?>"><?php _e('Network', 'wp-slideyournet' ) ?></label>
          <select class="widefat" name="<?php echo $this->get_field_name('network'); ?>" id="<?php echo $this->get_field_id('network'); ?>">
              <option value="" <?php selected('', $instance['network'], true, true); ?>><?php _e('All', 'wp-slideyournet' ) ?></option>
              <?php foreach($wp_syn->networks as $network): ?>
              <option value="<?php echo $network; ?>" <?php selected($network, $instance['network'], true, true); ?>><?php echo ucfirst($network); ?></option>
              <?php endforeach; ?>
          </select>
        </p>

        <p><label for="<?php echo $this->get_field_id('username'); ?>"><?php _e('Username', 'wp-slideyournet' ) ?></label>
          <input class="widefat" name="<?php echo $this->get_field_name('username'); ?>" id="<?php echo $this->get_field_id('username'); ?>" value="<?php echo $instance['username']; ?>"/>
        </p>

        <p><label for="<?php echo $this->get_field_id('search'); ?>"><?php _e('Search', 'wp-slideyournet' ) ?></label>
          <input class="widefat" name="<?php echo $this->get_field_name('search'); ?>" id="<?php echo $this->get_field_id('search'); ?>" value="<?php echo $instance['search']; ?>"/>
        </p>

        <p><label for="<?php echo $this->get_field_id('ref'); ?>"><?php _e('RePosts', 'wp-slideyournet' ) ?></label>
          <select class="widefat" name="<?php echo $this->get_field_name('ref'); ?>" id="<?php echo $this->get_field_id('ref'); ?>">
              <option value="" <?php selected('', $instance['ref'], true, true); ?>><?php _e('All', 'wp-slideyournet' ) ?></option>
              <option value="yes" <?php selected('yes', $instance['ref'], true, true); ?>><?php _e('Yes', 'wp-slideyournet' ) ?></option>
              <option value="no" <?php selected('no', $instance['ref'], true, true); ?>><?php _e('No', 'wp-slideyournet' ) ?></option>
          </select>
        </p>

        <hr>
        <h4><?php _e('Sort', 'wp-slideyournet' ) ?></h4>

        <p><label for="<?php echo $this->get_field_id('order_by'); ?>"><?php _e('Order by', 'wp-slideyournet' ) ?></label>
          <select class="widefat" name="<?php echo $this->get_field_name('order_by'); ?>" id="<?php echo $this->get_field_id('order_by'); ?>">
              <option value="pub_date" <?php selected('pub_date', $instance['order_by'], true, true); ?>><?php _e('Date', 'wp-slideyournet' ) ?></option>
          </select>
        </p>

        <p><label for="<?php echo $this->get_field_id('order'); ?>"><?php _e('Order', 'wp-slideyournet' ) ?></label>
          <select class="widefat" name="<?php echo $this->get_field_name('order'); ?>" id="<?php echo $this->get_field_id('order'); ?>">
              <option value="ASC" <?php selected('ASC', $instance['order'], true, true); ?>><?php _e('Ascendant', 'wp-slideyournet' ) ?></option>
              <option value="DESC" <?php selected('DESC', $instance['order'], true, true); ?>><?php _e('Descendant', 'wp-slideyournet' ) ?></option>
          </select>
        </p>

        <p><label for="<?php echo $this->get_field_id('per_page'); ?>"><?php _e('Posts per page', 'wp-slideyournet' ) ?></label>
          <input type="number" class="widefat" name="<?php echo $this->get_field_name('per_page'); ?>" id="<?php echo $this->get_field_id('per_page'); ?>" value="<?php echo $instance['per_page']; ?>"/>
        </p>

        <hr>
        <h4><?php _e('Display', 'wp-slideyournet' ) ?></h4>

        <p>
            <label for="<?php echo $this->get_field_id('display_date'); ?>">
                <input type="checkbox" <?php checked(1, $instance['display_date'], true); ?> name="<?php echo $this->get_field_name('display_date'); ?>" id="<?php echo $this->get_field_id('display_date'); ?>" value="1"/>
                <?php _e('Date', 'wp-slideyournet' ) ?>
            </label>
        </p>

        <p>
            <label for="<?php echo $this->get_field_id('display_username'); ?>">
                <input type="checkbox" <?php checked(1, $instance['display_username'], true); ?> name="<?php echo $this->get_field_name('display_username'); ?>" id="<?php echo $this->get_field_id('display_username'); ?>" value="1"/>
                <?php _e('Username', 'wp-slideyournet' ) ?>
            </label>
        </p>

        <p>
            <label for="<?php echo $this->get_field_id('display_avatar'); ?>">
                <input type="checkbox" <?php checked(1, $instance['display_avatar'], true); ?> name="<?php echo $this->get_field_name('display_avatar'); ?>" id="<?php echo $this->get_field_id('display_avatar'); ?>" value="1"/>
                <?php _e('Avatar', 'wp-slideyournet' ) ?>
            </label>
        </p>

        <p>
            <label for="<?php echo $this->get_field_id('display_media'); ?>">
                <input type="checkbox" <?php checked(1, $instance['display_media'], true); ?> name="<?php echo $this->get_field_name('display_media'); ?>" id="<?php echo $this->get_field_id('display_media'); ?>" value="1"/>
                <?php _e('Media', 'wp-slideyournet' ) ?>
            </label>
        </p>

        <p><label for="<?php echo $this->get_field_id('stylesheet'); ?>"><?php _e('Style sheet', 'wp-slideyournet' ) ?></label>
          <select class="widefat" name="<?php echo $this->get_field_name('stylesheet'); ?>" id="<?php echo $this->get_field_id('stylesheet'); ?>">
              <option value="wpsyn" <?php selected('wpsyn', $instance['stylesheet'], true, true); ?>><?php _e('WP SYN', 'wp-slideyournet' ) ?></option>
              <option value="none" <?php selected('none', $instance['stylesheet'], true, true); ?>><?php _e('None', 'wp-slideyournet' ) ?></option>
              <option value="theme" <?php selected('theme', $instance['stylesheet'], true, true); ?>><?php _e('wpsyn.css from current Theme', 'wp-slideyournet' ) ?></option>
          </select>
        </p>
       <?php
   }

}
