<?php
/*
 * Plugin Name: SlideYourNet
 * Plugin URI: https://wordpress.org/plugins/wp-slideyournet/
 * Description: SlideYourNet wrapper for WordPress
 * Version: 2.8
 * Author: NOUS Open Useful and Simple
 * Contributor: Bastien Ho, Sabrina Leroy, Vincent Merigeaux
 * Author URI: https://apps.avecnous.eu/?mtm_campaign=wp-plugin&mtm_kwd=wp-slideyournet
 * Text Domain: wp-slideyournet
 * Domain Path: /languages
 */

if(!class_exists('WP_SYN')){

class WP_SYN{
    var $networks;
    var $criteria;
    var $LocalContent;
    var $Settings;

    var $shortcode_instance = 0;

    var $is_network_installed;
    var $plugin_file_name;
    function __construct() {
        load_plugin_textdomain('wp-slideyournet');

        add_action('init', array(&$this, 'init'), 10);

        add_action('widgets_init', array(&$this, 'register_widgets'),1,1);

        add_filter('wp_syn_get_thumbnail_url', array(&$this, 'thumbnail_url'), 10, 3);
        add_filter('wp_syn_get_thumbnail_url', array(&$this, 'thumbnail_url_proxy'), 20, 3);
        add_filter('wp_syn_image_proxy_size', array(&$this, 'image_proxy_size'), 10, 2);
        add_filter('wp_syn_thumbnail_size_local_storage', array(&$this, 'thumbnail_size'), 10, 3);


        add_shortcode('syn', array(&$this, 'shortcode'));

        include_once plugin_dir_path(__FILE__) . 'inc/utils.php';
        include_once plugin_dir_path(__FILE__) . 'inc/settings.php';
        $this->Settings = new WP_SYN\SETTINGS;
        include_once plugin_dir_path(__FILE__) . 'inc/local-content.php';
        $this->LocalContent = new WP_SYN\LOCAL_CONTENT;

        $this->plugin_file_name = basename(__DIR__).'/'.basename(__FILE__);

        include_once plugin_dir_path(__FILE__) . 'inc/blocks.php';
    }
    // PHP 4 loader
    function WP_SYN(){
        $this->__construct();
    }
    /**
     * Localize main datas
     */
    function localize(){
        __('WP SlideYourNet', 'wp-slideyournet');
        __('SlideYourNet wrapper for WordPress', 'wp-slideyournet');
    }

    function init(){
        $img_proxy_query = filter_input(INPUT_GET, 'syn_img');
        if($img_proxy_query){
          $this->image_proxy($img_proxy_query, 'media');
        }
        $avatar_proxy_query = filter_input(INPUT_GET, 'syn_av');
        if($avatar_proxy_query){
          $this->image_proxy($avatar_proxy_query, 'avatar');
        }
        if(class_exists('widget_home_builder')){
            include_once (plugin_dir_path(__FILE__) . 'inc/widget-home-builder.php');
        }
        // Makes sure the plugin is defined before trying to use it
        if ( ! function_exists( 'is_plugin_active_for_network' ) ) {
            require_once( ABSPATH . '/wp-admin/includes/plugin.php' );
        }
        $plugin_name = basename(__DIR__).'/'.basename(__FILE__);
        $this->is_network_installed = is_plugin_active_for_network( $plugin_name );
        if ($this->is_network_installed) {
            add_action('network_admin_menu', array(&$this, 'network_admin_menu'));

            add_action(
                'network_admin_edit_wpsynmu',
                array($this, 'trigger_wpsynmu')
            );
        }

        $this->networks = apply_filters('wpsyn_networks', array(
           'twitter',
           'facebook',
           'instagram'
        ));

        $this->criteria = array(
                // Misc
                'title'=>'',
                'className'=>'',
                'layout'=>'',
                'per_column'=>3,

                // Account
                'session'=>null,
                'token'=>null,

                // Filters
                'network'=>'',
                'username'=>'',
                'search'=>'',
                'ref'=>'',

                // Sortering
                'order_by'=>'pub_date',
                'order'=>'DESC',
                'per_page'=>5,
                'characters'=>0,

                // Displaying
                'display_date'=>'',
                'display_username'=>'',
                'display_avatar'=>'',
                'display_media'=>'',
                'stylesheet'=>'wpsyn',
                'local_storage'=>'',
            );
    }


    public function register_widgets(){
        require_once (plugin_dir_path(__FILE__) . 'widget.php');
        register_widget('WP_SYN_Widget');
    }

    /**
     *
     * @param array $_args
     * @return array
     */
    public function get_posts($_args){
        $result = [];
        $args = wp_parse_args( (array) $_args, $this->criteria );
        if(isset($args['session']) && isset($args['token']) && $args['session'] && $args['token']){
            $args['local_storage'] = 0;
        }
        if(isset($args['local_storage']) && $args['local_storage'] == 1){

          // ref
          $query_args = [
              'post_type'  => $this->LocalContent->cpt_syn_post ,
            ];
          $meta_query = [
              'relation' => 'AND',
            ];

          if(isset($args['network']) && !empty($args['network'])){
            $meta_query[] = [
              'key'     => 'syn_network',
              'value'   => $args['network'],
              'compare' => '=',
            ];
          }
          if(isset($args['username']) && !empty($args['username'])){
            $meta_query[] = [
              'key'     => 'syn_username',
              'value'   => $args['username'],
              'compare' => 'LIKE',
            ];
          }
          if(isset($args['search']) && !empty($args['search'])){
            $query_args['s'] = $args['search'];
          }
          if(isset($args['order_by']) && !empty($args['order_by'])){
            $query_args['orderby'] = $args['order_by'];
          }
          if(isset($args['order']) && !empty($args['order'])){
            $query_args['order'] = $args['order'];
          }
          if(isset($args['per_page']) && !empty($args['per_page'])){
            $query_args['posts_per_page'] = $args['per_page'];
          }
          $query_args["meta_query"] = $meta_query;

          $query = new WP_Query( apply_filters('wpsyn_get_posts_params_local_storage', $query_args) );

          foreach($query->posts as $post){
            $syn_rawdata_uptodate = get_post_meta( $post->ID, 'syn_rawdata_uptodate', true);
            if(!$syn_rawdata_uptodate){
              $syn_rawdata_uptodate = get_post_meta( $post->ID, 'syn_rawdata', true);
            }
            if($syn_rawdata_uptodate){
              $result[] = json_decode(json_encode($syn_rawdata_uptodate,FALSE));
            }
          }

        }
        else{

          $params = array(
              's'=>$this->Settings->Options()['name'],
              't'=>$this->Settings->Options()['token'],
              'object'=>'post',
              'type'=>'get'
          );

          foreach ($args as $arg=>$value){
              if(!empty($value)){
                  $params[$arg]=$value;
              }
          }

          // If Session is overriden, cleanup params
          if(isset($params['session']) && isset($params['token']) && $params['session'] && $params['token']){
               $params['s'] = $params['session'];
               $params['t'] = $params['token'];
               unset($params['session']);
               unset($params['token']);
          }

          $params = apply_filters('wpsyn_get_posts_params', $params);

          $url = add_query_arg($params, $this->Settings->Options()['url']);
          $cache_key = md5($url);
          $cache_key_stamp = $cache_key.'_stamp';
          $cache = $this->Settings->Options()['cache'];

          $result = get_transient($cache_key);
          $result_age = get_transient($cache_key_stamp);
          if(!$result_age || !$result || !$cache || (is_array($result) && !count($result))){
              $temp_extract = \WP_SYN\UTILS::get_remote_content($url);
              $temp_result = $temp_extract ? (array) json_decode($temp_extract) : false;
              if(!is_numeric($cache)){
                  $cache = 15;
              }
              if($cache){
                  if($temp_result && !empty($temp_result) && $temp_result!=$result){
                    set_transient($cache_key, $temp_result, 0);
                  }
                  set_transient($cache_key_stamp, time(), $cache*MINUTE_IN_SECONDS);
              }
              else{
                if($result){
                  delete_transient($cache_key);
                }
                if($result_age){
                  delete_transient($cache_key_stamp);
                }
              }
              if($temp_result){
                $result = $temp_result;
              }
          }

        }
        return apply_filters('wpsyn_posts', $result);

    }

    /**
     * Returns a post content after parsing URLs and hashtags
     * @param object $post
     * @return string
     */
    function content($post, $instance = false){
        $content = $post->content;

        if($instance && isset($instance['characters']) && !empty($instance['characters']) && $instance['characters'] != 0){
          if(strlen($content) > $instance['characters']){
            $content = substr($content,0, $instance['characters']). " …";
          }
        }
        $medias_scheme = apply_filters('wpsyn_medias_scheme', array(
            'twitter'=>array(
                '#'=>'https://twitter.com/hashtag/%s',
                '@'=>'https://www.twitter.com/%s',
            ),
            'instagram'=>array(
                '#'=>'https://instagram.com/explore/tags/%s',
                '@'=>'https://www.instagram.com/%s',
            ),
            'facebook'=>array(
                '#'=>'https://www.facebook.com/hashtag/%s',
                '@'=>'https://www.facebook.com/%s',
            )
        ));
        if(isset($medias_scheme[$post->network])){
            //Convert urls to <a> links
            $content = preg_replace("/([\w]+\:\/\/[\w\-\?\&\;\#\~\=\.\/\@]+[\w\/])/u", '<a target="_blank" href="$1" class="wpsyn-link">$1</a>', $content);
            //Convert hashtags to twitter searches in <a> links
            $content = preg_replace("/(^|[ \.\,\;\:\s])#([\w0-9\/\._]*)/u", '$1<a target="_blank" href="'.sprintf($medias_scheme[$post->network]['#'], '$2').'" class="wpsyn-hastag">#$2</a>', $content);
            //Convert attags to twitter profiles in <a> links
            $content = preg_replace("/(^|[ \.\,\;\:])@([\w0-9\._]*)/u", '$1<a target="_blank" href="'.sprintf($medias_scheme[$post->network]['@'], '$2').'" class="wpsyn-mention">@$2</a>', $content);
            //Convert pic.twitter to image icon
            $content = preg_replace("/(pic.twitter.com\/[A-Za-z0-9\/\.]*)/u", '<a href="https://$1" target="_blank" class="wpsyn-mention"><span class="dashicons dashicons-image"></span><span class="screen-reader-text">'.__('Image', 'wp-slideyournet').'</span></a>', $content);
        }

        // Beautyfull typo rules
        if(strstr('fr', get_locale())){
            $content = preg_replace('/\s(\;|\:|\?|\!|\&raquo\;|\"|\»)/', '&nbsp;$1', $content);
            $content = preg_replace('/(\&laquo\;|\«|\s[\w]{1,2})\s/', '$1&nbsp;', $content);
        }

        return apply_filters('wpsyn_post_content', $content);
    }

    function thumbnail_url($url, $network, $type='media'){
        if($network=='facebook'){
            $n_url = parse_url($url);
            parse_str($n_url['query'], $query);
            if(isset($query['url'])){
                return $query['url'];
            }
        }
        return $url;
    }

    function thumbnail_url_proxy($url, $network, $type='media'){
      if($url!=''){
        $key = $type=='media' ? 'syn_img' : 'syn_av';
        $url=add_query_arg(
           array(
             $key=>base64_encode($url),
           ),
           site_url()
        );
      }
    	return $url;
    }

    function thumbnail_size($size, $network, $instance){
      if($instance['stylesheet'] == "masonry"){
        $size = "medium";
      }
      return $size;
    }
    function media_get_extension($url){
      $params = parse_url($url);
      return substr($params['path'], -3);
    }

    function image_proxy_size($sizes, $type){
      if($type=='avatar'){
        $sizes = array('w'=>36, 'h'=>36, 'q'=>70);
      }
      return $sizes;
    }

    function image_proxy($url, $type='media'){
      $cache = $this->Settings->Options()['cache'];
      $hash = $url;
      $url = base64_decode($hash);
      $upload_dir = wp_upload_dir();
      $local_dir = $upload_dir['basedir'].'/slideyournet';
      if ( ! is_dir( $local_dir ) ) {
          wp_mkdir_p( $local_dir );
      }
      $local_path = $local_dir.'/'.md5($hash).'.jpg';

      if(!is_file($local_path)){
        $canvas_size = apply_filters('wp_syn_image_proxy_size', array('w'=>450, 'h'=>450, 'q'=>75), $type);
        $ext = $this->media_get_extension($url);
        $size = getimagesize($url);
        $src = false;
        if($ext=='gif'){ $src = imagecreatefromgif($url); }
        elseif($ext=='jpg' || $ext=='peg'){ $src = imagecreatefromjpeg($url); }
        elseif($ext=='png'){ $src = imagecreatefrompng($url); }

        if($size && $src){
          $neww=$size[0];
          $newh=$size[1];
          $ratio = 1;
          $jeveuxW = $neww;
          $jeveuxH = $newh;
          if(isset($canvas_size["w"])){
            $jeveuxW = $canvas_size["h"];
            if($neww >= $jeveuxW){
              $ratio = $neww/$jeveuxW;
            }
          }
          elseif(isset($canvas_size["w"])){
            $jeveuxH = $canvas_size["h"];
            if($newh > $jeveuxH){
              $ratio = $newh/$jeveuxH;
            }
          }
          $neww /= $ratio;      $newh /= $ratio;
          $des = imagecreatetruecolor ($neww, $newh) ;
          $fond = imagecolorallocate($des,  255, 255, 255);

          if($des && $src){
            imagecopyresampled( $des, $src, 0, 0, 0, 0, $neww, $newh, $size[0], $size[1]);
            imagejpeg($des, $local_path, $canvas_size['q']);
            imagedestroy($des);
            $ext = 'jpg';
          }
        }
      }
      if(is_file($local_path)){
        $img = file_get_contents($local_path);
        $ext = $this->media_get_extension($local_path);
      }
      else{
        $img = file_get_contents($url);
      }
      header("Content-Type: image/$ext");
      print $img;
      exit;
    }


    function shortcode($atts, $content=''){
        $instance = apply_filters('wpsyn_shortcode_params', shortcode_atts($this->criteria, $atts));
        $posts = $this->get_posts($instance);
        if(!$posts || !count($posts)){
            return apply_filters('wpsyn_shortcode_html', $content, $instance, $content);
        }
        $this->shortcode_instance++;

        $class = array(
            'wp-syn-wrapper',
            $instance['className'],
            $instance['layout'],
            'per-column-'.$instance['per_column'],
        );

        $html = '<div id="wp-syn-instance-'.$this->shortcode_instance.'" data-id="'.$this->shortcode_instance.'" class="'.esc_attr(implode(' ', $class)).'">';

        ob_start();
        include (plugin_dir_path(__FILE__) . 'views/list.php');
        $html.=ob_get_clean();

        $html.='</div><!--.wp-syn-wrapper-->';

        return apply_filters('wpsyn_shortcode_html', $html, $instance, $content);
    }
}
global $wp_syn;
$wp_syn = new WP_SYN();

function wp_syn(){
    global $wp_syn;
    return $wp_syn;
}

}
